# python
########################################################
# gct2IGCcompat.py
# Author: Kevin Ying
# Date:  14 April 2011 

# Make a GCT file compatible with the Integrative Genomics Viewer (IGV) 
# Assumes the Description column of the GCT starts with the gene symbol 
# Appends the IGV readable format to the description 
# The description field needs to start with the {geneSymbol} followed by ':'
#
# eg.
# DESCRIPTION COLUMN
    # {geneSymbol}
	# OR
    # {geneSymbol}: na
    # OR
    # {geneSymbol}:: na
# BECOMES
    # {geneSymbol} |@{geneSymbol}|
	# OR
    # {geneSymbol}: na |@{geneSymbol}|
    # OR
    # {geneSymbol}:: na |@{geneSymbol}|
########################################################

########################################################
# Version History
# 1:
#	- 
# 2.3 
#	- Doesnt fail on missing data now
########################################################

# command line:
# python gct2IGVcompat.py -g <gct.file> -o <output.file>

# GenePattern command line:
# python <libdir>gct2IGVcompat.py <gct.file> <output.file>

########################################################
# Constants
#
# Column indices of a 2D array corresponding to the columns of a GCT
# For info on the GCT file format see
# http://www.broadinstitute.org/cancer/software/genepattern/tutorial/gp_fileformats.html#gct
GCT_NAME_IDX = 0 # unique row identifier, usually probe ID
GCT_DESC_IDX = 1 # Description
########################################################

########################################################
# Imports
import sys
import os
from optparse import OptionParser
########################################################

########################################################
# MAIN
########################################################
def main():
	usage =  "Usage: %s -g <gct file> -o <output file>" % os.path.basename(sys.argv[0])
	parser = OptionParser(usage=usage)
	parser.add_option("-g", "--gctFile",
			  action="store", type="string", dest="gctFile",
			  help="The GCT file to be modified")
	parser.add_option("-o", "--outFile",
			  action="store", type="string", dest="outFile",
			  help="The name of the IGV compatible GCT file to write to")
	(options, args) = parser.parse_args()

	gctFile  = options.gctFile
	outFile  = options.outFile

	# Open output file handle
	outf = open(outFile, 'w')

	# Store GCT file 
	(header, origGCT)  = getOrigGCTList(gctFile)

	#########################################
	# Begin modifying and printing the GCT
	#########################################

	# Print header first
	for i in header:
		print >>outf, "\t".join(i)
		
	# Loop through gct and modify as necessary
	notFound = 0
	for i in range(len(origGCT)):
		#print originalGCTlist[i]
		if origGCT[i][GCT_DESC_IDX]:
			# Split and take the geneSymbol from the Description
			# Assumes the Description is of the format:
			# {geneSymbol}: rest of descripion
			geneSymbol = getGeneSymbol(origGCT[i][GCT_DESC_IDX])
			# Append the geneSymbol to the end of the description as
			# |@geneSymbol|
			origGCT[i][1] = origGCT[i][GCT_DESC_IDX] + ' |@' + geneSymbol + '|'
		else:
			notFound = notFound + 1
		print >>outf, "\t".join(origGCT[i])
	
	if notFound > 0:
		print "WARNING: " + (str(notFound) + " out of " + str(len(origGCT)) + " rows did not have a description\n")	

	# Close output file handle
	outf.close() 

########################################################
# read a gct file
#
# returns a 2D array of the file header and gct data
# For more details:
# http://www.broadinstitute.org/cancer/software/genepattern/tutorial/gp_fileformats.html#gct
#
# eg. of a GCT file, the first three lines is the header 
#
# #1.2
# 1000	130
# Name	Description	Sample1	Sample2	Sample3
# 123_a	AGRN: agrin 	15.13	123.123	123.4	
# 124_b	ESPN: espin 	16.13	124.123	124.5	
#
# 
#
# input: location of gct file
# ouput: (headers, gctlist)
########################################################
def getOrigGCTList(file):
	# Open and save gct file
	originalGCTlist = []
	f = open(file, 'rU') # "rU" allows for reading from unix, windows and mac formats
	
	# parse GCT file data and headers
	version = f.next().strip().split("\t")
	nrowSample = f.next().strip().split("\t")
	colnames = f.next().strip().split("\t")
	headers = [version, nrowSample, colnames]
	
	for line in f:
		line = line.strip()
		line = line.split("\t")
		#if len(line) == (len(colnames)): # Rudimentary checking of line format
			#print line
		originalGCTlist.append(line)
		#else:
			#sys.stderr.write("GCTFile: line ignore, bad format:" + str(line) + "\n")
			#print ("GCTFile: line ignore, bad format:" + str(line) + "\n")
	f.close()
	if len(originalGCTlist) != int(nrowSample[0]):
		sys.stderr.write("Warning: " + str(nrowSample[0]) + " does not match actual no. of rows " + str(len(originalGCTlist)) + '\n')
	return (headers, originalGCTlist)

########################################################
# Helper functions
########################################################
def getGeneSymbol(description):
	geneSymbol = description.split(':')[0].strip()
	return geneSymbol
	

########################################################
# run main 
########################################################
if __name__ == "__main__":
	main()
